<?php
/*
Plugin Name: Photopress
Plugin URI: http://familypress.net/photopress/
Description: Adds user-friendly image handling tools to Wordpress, including a popup upload and browse tool, a random image function, and a simple album. Installs and uses a database table.
Version: 0.8.1
Author: Isaac Wedin
Author URI: http://familypress.net/
*/

/* INSTALLATION
Extract the archive in your WP plugins folder. Create a 'photos' folder in your wp-content folder and make it writable. Activate the plugin (or de-activate and re-activate if you're upgrading), which will install a new database table and import any old data. Configure the options at Options:Photopress. If you're using permalinks make sure to update your permalink structure at Options:Permalinks. Enter information about your photos during upload or at Manage:Photopress. Edit pp_album.php and pp_album_css.php to work with your theme if necessary (they're designed to work well with the default theme).
*/

/*  Copyright 2004, 2005  Isaac Wedin (email : isaac@familypress.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Include shared functions and defaults
require_once(ABSPATH . 'wp-content/plugins/photopress/include.php');

// Inserts Javascript to launch the Photos button
function pp_popup_js() {
	echo '
<script language="JavaScript" type="text/javascript"><!--
function pp_popup() {
	var mylink = "' . get_settings('siteurl') . '/wp-content/plugins/photopress/popup.php?action=upload";
	var windowname = "photopress";
	if (! window.focus) return true;
		var href;
	if (typeof(mylink) == "string")
		href = mylink;
	else
		href = mylink.href;
	window.open(href, windowname, "width=400,height=600,scrollbars=yes");
	return false;
}
function setbrowsesort(sort) {
	window.top.location.href = "' . get_settings('siteurl') . '/wp-admin/edit.php?page=photopress.php&" + sort.options[sort.selectedIndex].value;
	return false;
}
//--></script>
	';
}

add_action('admin_head','pp_popup_js');

// Adds the Photos button to the Quicktags menu. Borrowed from ContactForm, which I think borrowed from Alex King's JS Quicktags.
function pp_insert_button() {
	echo '
<script language="JavaScript" type="text/javascript"><!--
	document.getElementById("quicktags").innerHTML += "<input type=\"button\" class=\"ed_button\" id=\"ed_upload\" value=\"Photos\" onclick=\"return pp_popup();\" />";
//--></script>
	';
}

add_action('admin_footer', 'pp_insert_button');

// random image function. given $category and $type it returns a random image, formatted by $type: 1 = linked thumb, 2 = unlinked thumb, 3 = file name; CSS class can be set by passing a third argument (did this to get proper class on the main album page)
function pp_random_image($category='', $type = 1, $style = 'random') {
	global $pp_options, $table_prefix, $wpdb;
	$table_name = $table_prefix . 'photopress';
	if (empty($category)) {
		$random_image_array = $wpdb->get_results("SELECT imgfile,imgname FROM $table_name ORDER BY RAND() LIMIT 1",ARRAY_A);
	} else {
		$category = $wpdb->escape($category);
		$random_image_array = $wpdb->get_results("SELECT imgfile,imgname FROM $table_name WHERE binary imgcat='$category' ORDER BY RAND() LIMIT 1",ARRAY_A);
	}
	$random_image = $random_image_array[0]['imgfile'];
	if (!empty($random_image[0]['imgname'])) {
		$random_alt_text = $random_image_array[0]['imgname'];
	} else {
		$random_alt_text = str_replace("_", " ", substr($random_image, 0, -4));
	}
	$random_image_thumb = $pp_options['thumbprefix'] . $random_image;
	$random_image_thumb_size = getimagesize( $pp_options['photospath'] . '/' . $random_image_thumb);
	$random_thumb = '<img class="';
	if ($style == 'album') {
		$random_thumb .= 'pp_centered';
	} elseif ($style == 'random') {
		$random_thumb .= $pp_options['rand_class'];
	} else {
		$random_thumb .= $style;
	}
	$random_thumb .= '" src="' . $pp_options['photosaddress'] . '/' . $random_image_thumb . '" ' . $random_image_thumb_size[3] . ' alt="' . $random_alt_text . '" title="' . $random_alt_text . '" />';
	if ($pp_options['album'] == '1') {
		$random_linked_thumb =  '<a href="' . htmlentities2($pp_options['imgaddress'] . $random_image) . '" title="' . $random_alt_text . '">' . $random_thumb . '</a>';
	} else {
		$random_image_address = $pp_options['photosaddress'] . '/' . $random_image;
		$random_image_size = getimagesize($pp_options['photospath'] . '/' . $random_image);
		$random_linked_thumb =  '<a href="' . htmlentities2($random_image_address) . '" onclick="pp_image_popup(\'' . $random_image_address . '\',' . $random_image_size[0] . ',' . $random_image_size[1] . '); return false;" title="' . $random_alt_text . '" target="_blank">' . $random_thumb . '</a>';
	}
	if ($type == 3) {
		return $random_image;
	} elseif ($type == 2) {
		return $random_thumb;
	} else {
		return $random_linked_thumb;
	}
}

// template function to make a random linked thumb
function pp_random_image_bare() {
	global $wpdb, $table_prefix;
	$table_name = $table_prefix . 'photopress';
	echo pp_random_image();
}

// put a random link to the photo album and/or a random thumbnail in the meta section of the sidebar, depending on the options, does nothing if there aren't any images yet
function pp_stuff_in_meta() {
	global $pp_options;
	$stuff_for_meta = '';
	if ($pp_options['meta_link'] == 1) {
		$stuff_for_meta .= '<li><a href="' . $pp_options['albumaddress'] . '">' . __('Photo Album','photopress') . '</a></li>' . "\n";
	}
	if ($pp_options['meta_rand'] == 1 && pp_count()) {
		$stuff_for_meta .= '<li>' . pp_random_image() . "</li>\n";
	}
	echo $stuff_for_meta;
}

add_action('wp_meta', 'pp_stuff_in_meta');

// wrapper function to display the album
function pp_album() {
	global $pp_options;
	echo '<h2><a href="' . $pp_options['albumaddress'] . '">' . __('Photo Album','photopress') . '</a></h2>';
	if (isset($_GET['pp_image'])) {
		$image = stripslashes($_GET['pp_image']);
		pp_display_image($image);
	} elseif (isset($_GET['pp_cat'])) {
		$cat = stripslashes($_GET['pp_cat']);
		pp_display_cat($cat);
	} else {
		pp_display_main();
	}
}

// displays images from $category in an html table
function pp_display_cat($category) {
	global $pp_options;
	$category = stripslashes($category);
	echo '<h3>' . $category . '</h3>
	';
	if (isset($_GET['pp_page'])) {
		$current_page = $_GET['pp_page'];
	} else {
		$current_page = 1;
	}
	if (isset($_GET['sort'])) {
		$sort = $_GET['sort'];
	} else {
		$sort = 'imgfile';
	}
	$column = 0;
	$image_count = pp_count($category);
	$pages = (int)ceil($image_count/$pp_options['images_per_page']);
	if ($pages > 1) {
		echo '<p>
		';
		for ($i=1; $i<=$pages; $i++) {
			if ($i != $current_page) {
				echo '<a href="' . htmlentities2($pp_options['cataddress'] . $category . $pp_options['page_token'] . $i) . '">';
			}
			echo __('Page','photopress') . ' ' . $i;
			if ($i != $current_page) {
				echo '</a>';
			}
			if ($i < $pages) {
				echo ' | ';
			}
		}
		echo '</p>
		';
		$list_start = ($current_page - 1) * $pp_options['images_per_page'];
	} else {
		$list_start = 0;
	}
	$cat_for_query = addslashes($category);
	$cat_images = pp_images_with_data($cat_for_query,$sort,$list_start,$pp_options['images_per_page']);
	echo '<table id="pp_gallery">
	';
	foreach((array)$cat_images as $key=>$array_img) {
		$column++;
		if($column == 1) {
			echo '<tr>';
		}
		$thumbsize = getimagesize($pp_options['photospath'] . '/' . $pp_options['thumbprefix'] . $array_img['imgfile']);
		if (!empty($array_img['imgname'])) {
			$cleanedname = stripslashes($array_img['imgname']);
		} else {
			$cleanedname = strtr(substr($array_img, 0, strrpos($array_img['imgfile'], '.')), "_", " ");
		}
		echo '<td class="pp_cell"><a href="' . htmlentities2($pp_options['imgaddress'] . $array_img['imgfile']) . '" title="' . $cleanedname . '"><img src="' . $pp_options['photosaddress'] . '/' . $pp_options['thumbprefix'] . $array_img['imgfile'] . '" ' . $thumbsize[3] . ' class="pp_centered" alt="' . $cleanedname . '" /><br />' . $cleanedname . '</a></td>';
		if($column == $pp_options['album_columns']) {
			echo '</tr>
			';
			$column = 0;
		}
	}
	if ($column < $pp_options['album_columns'] && $column > 0) {
		echo '</tr>
		';
	}
	echo '</table>
	';
}

function pp_display_main() {
	global $pp_options;
	$categories = array();
	$categories = pp_list_cats();
	$column = 0;
	echo '<table id="pp_gallery">
	';
	if (count($categories) > 1) {
		foreach((array)$categories as $cat) {
			$column++;
			if ($column == 1) {
				echo '<tr>';
			}
			$image_count = pp_count($cat);
			echo '<td class="pp_cell"><a href="' . htmlentities2($pp_options['cataddress'] . stripslashes($cat)) . '" title="' . stripslashes($cat) . '">' . pp_random_image($cat,2,'album') . '<br />' . stripslashes($cat) . ' (' . $image_count . ')</a></td>';
			if ($column == $pp_options['album_columns']) {
				echo '</tr>
				';
				$column = 0;
			}
		}
		if ($column < $pp_options['album_columns'] && $column > 0) {
			echo '</tr>
			';
		}
	} elseif (count($categories) == 1) {
		echo '<tr>';
		$cat_images = pp_cat_images($categories[0]);
		$image_count = count($cat_images);
		echo '<td class="pp_cell"><a href="' . htmlentities2($pp_options['cataddress'] . stripslashes($categories[0])) . '" title="' . stripslashes($categories[0]) . '">' . pp_random_image((array)$cat_images,2) . '<br />' . stripslashes($categories[0]) . ' (' . $image_count . ')</a></td>';
		echo '</tr>
		';
	} else {
		echo '<tr><td><strong>' . __('No photos uploaded yet','photopress') . '</strong></td></tr>
		';
	}
	echo '</table>
	';
}

// displays an image, linked to a larger version or to the original if it's there
function pp_display_image($image) {
	global $pp_options;
	if (file_exists($pp_options['photospath'] . '/' . $image)) {
		if (pp_get_data($image)) {
			$image_data = pp_get_data($image);
			$image_cat = stripslashes($image_data['imgcat']);
			$cleanedname = stripslashes($image_data['imgname']);
			$imgdesc = stripslashes($image_data['imgdesc']);
		} else {
			$image_cat = 'Default';
			$cleanedname = strtr(substr($image, 0, -4), "_", " ");
		}
		echo '<div id="pp_meta"><strong><a href="' . htmlentities2($pp_options['cataddress'] . stripslashes($image_cat)) . '">' . $image_cat . '</a> : ' . $cleanedname . '</strong></div>' . "\n";
		$cat_array = pp_cat_images($image_cat);
		$image_size = getimagesize($pp_options['photospath'] . '/' . $image);
		$itemcount = sizeof($cat_array);
		if ($itemcount > 1) {
			$prevnext = '<div id="pp_prevnext">' . "\n";
			for ($j = 0; $j < $itemcount; $j++) {
				$item = $cat_array[$j];
				if ($image == $item) {
					if ($cat_array[$j-1]) {
						$prevnext .= '<a class="pp_prev" href="' . htmlentities2($pp_options['imgaddress'] . $cat_array[$j-1]) . '">' . __('Previous','photopress') . '</a>' . "\n";
					} else {
						$prevnext .= '<div class="pp_prev"></div>' . "\n";
					}
					if ($cat_array[$j+1]) {
						$prevnext .= '<a class="pp_next" href="' . htmlentities2($pp_options['imgaddress'] . $cat_array[$j+1]) . '">' . __('Next','photopress') . '</a>' . "\n";
					} else {
						$prevnext .= '<div class="pp_next"></div>' . "\n";
					}
				}
			}
			$prevnext .= "</div>\n";
			echo $prevnext;
		}
		echo '<div id="pp_gallery">';
		if (is_file($pp_options['photospath'] . '/' . $pp_options['origprefix'] . $image)) {
			$orig_size = getimagesize($pp_options['photospath'] . '/' . $pp_options['origprefix'] . $image);
			echo '<a href="' . htmlentities2($pp_options['photosaddress']) . '/' . $pp_options['origprefix'] . $image . '" onclick="pp_image_popup(\'' . $pp_options['photosaddress'] . '/' . $pp_options['origprefix'] . $image . '\',' . $orig_size[0] . ',' . $orig_size[1] . '); return false;" title="' . $cleanedname . '" target="_blank"><img class="pp_centered" src="' . $pp_options['photosaddress'] . '/' . $image . '" ' . $image_size[3] . ' alt="' . $cleanedname . '" /></a>';
		} else {
			echo '<a href="' . htmlentities2($pp_options['photosaddress']) . '/' . $image . '" onclick="pp_image_popup(\'' . $pp_options['photosaddress'] . '/' . $image . '\',' . $image_size[0] . ',' . $image_size[1] . '); return false;" title="' . $cleanedname . '" target="_blank"><img class="pp_centered" src="' . $pp_options['photosaddress'] . '/' . $image . '" ' . $image_size[3] . ' alt="' . $cleanedname . '" /></a>';
		}
		if (!empty($imgdesc)) {
			echo '<p>' . $imgdesc . '</p>';
		}
		if ( $pp_options['show_posts'] == '1' ) {
			query_posts("s=$image");
			if ( have_posts() ) :
				echo '<p><strong>' . __('Posts with this image','photopress') . ':</strong></p>
			<ul>';
			while ( have_posts() ) : the_post();
				echo '<li><a href="';
				the_permalink();
				echo '">';
				the_title();
				echo '</a></li>';
			endwhile;
			echo '</ul>';
		else: endif;
		}
		echo "</div>\n";
	} else {
		echo "<p>" . __("Image not found, probably deleted.","photopress") . "</p>\n";
	}
}

function pp_album_template() {
	if ($_GET['pp_album']) {
		if ( file_exists(TEMPLATEPATH . 'pp_album.php') ) {
			include(TEMPLATEPATH . 'pp_album.php');
		} else {
			include(ABSPATH . 'wp-content/plugins/photopress/pp_album.php');
			exit;
		}
	}
	return;
}

add_action('template_redirect','pp_album_template');

// insert the css for the album and the javascript to do image popups
function pp_album_css() {
	if ($_GET['pp_album']) {
		if (file_exists(TEMPLATEPATH . 'pp_album_css.php')) {
			include(TEMPLATEPATH . 'pp_album_css.php');
		} elseif (file_exists(ABSPATH . 'wp-content/plugins/photopress/pp_album_css.php')) {
			include(ABSPATH . 'wp-content/plugins/photopress/pp_album_css.php');
		}
	}
	echo '
<script language="JavaScript" type="text/javascript"><!--
function pp_image_popup(image,width,height) {
	height += 24;
	width += 24;
	image_popup = window.open(image,\'image\',\'height=\' + height + \',width=\' + width + \',toolbar=no,menubar=no,scrollbars=yes,resizable=yes\');
	image_popup.resizeTo(width+2,height+30);
	image_popup.focus();
	return false;
}
	//--></script>';
}

add_action('wp_head','pp_album_css');

// do the Options page, setting the min level to use it at 8
function pp_add_options_page() {
	add_options_page('Photopress', 'Photopress', 8, basename(__FILE__), 'pp_options_subpanel');
}

function pp_options_subpanel() {
	global $pp_options;
	if (isset($_POST['pp_options_update'])) {
	   $pp_updated_options = array();
		$pp_updated_options = $_POST;
		add_option('pp_options');
		update_option('pp_options', $pp_updated_options);
		$pp_options = get_option('pp_options');
		echo '<div class="updated">' . __('Photopress options updated.','photopress') . '</div>';
	}
	echo '
	<div class="wrap">
	<h2>' . __('Photopress Options','photopress') . '</h2>
	<form name="pp_options" method="post">
	<input type="hidden" name="pp_options_update" value="update" />
	<fieldset class="options">
	<table width="100%" cellspacing="2" cellpadding="5" class="editform">
		<tr>
			<th scope="row">' . __('Path to your photos folder:','photopress') . '</th>
			<td><input name="photospath" type="text" id="photospath" value="' . $pp_options['photospath'] . '" size="40" /><br />' . __('Best guess:','photopress') . ' "' . ABSPATH . '/wp-content/photos" ' . __('Make sure this path is correct and that the folder at the end exists and is writable.','photopress') . '</td>
			</td>
		</tr>
		<tr>
			<th scope = "row">' . __('Wordpress address:','photopress') . '</th>
			<td><input name="wpaddress" type="text" id="wpaddress" value="' . $pp_options['wpaddress'] . '" size="40" /><br />' . __('Best guess:','photopress') . ' "' . get_settings('siteurl') . '"</td>
		</tr>
		<tr>
			<th scope = "row">' . __('Address of your photos folder:','photopress') . '</th>
			<td><input name="photosaddress" type="text" id="photosaddress" value="' . $pp_options['photosaddress'] . '" size="40" /><br />' . __('Recommended:','photopress') . ' "' . get_settings('siteurl') . '/wp-content/photos"</td>
		</tr>
		<tr>
			<th scope="row">' . __('Use permalinks:','photopress') . '</th>
			<td>
			<label><input name="use_permalinks" type="radio" value="1" ';
			if ($pp_options['use_permalinks'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label><br />
			<label><input name="use_permalinks" type="radio" value="0" ';
			if ($pp_options['use_permalinks'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />
			</td>
		</tr>
		<tr>
			<th scope = "row">' . __('Album token for permalinks:','photopress') . '</th>
			<td><input name="album_token" type="text" id="album_token" value="' . $pp_options['album_token'] . '" size="10" /><br />' . __('This will appear just after your blog address; use something like "album". (Ignored if not using permalinks.)','photopress') . '</td>
		</tr>
		<tr>
			<th scope = "row">' . __('Category token for permalinks:','photopress') . '</th>
			<td><input name="cat_token" type="text" id="cat_token" value="' . $pp_options['cat_token'] . '" size="10" /><br />' . __('This will appear just after the album token; use something like "cat". (Ignored if not using permalinks.)','photopress') . '</td>
		</tr>
		<tr>
			<th scope = "row">' . __('Images token for permalinks:','photopress') . '</th>
			<td><input name="images_token" type="text" id="images_token" value="' . $pp_options['images_token'] . '" size="10" /><br />' . __('This will appear just after the album token; use something like "images". (Ignored if not using permalinks.)','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Keep original images:','photopress') . '</th>
			<td>
			<label><input name="originals" type="radio" value="1" ';
			if ($pp_options['originals'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label><br />
			<label><input name="originals" type="radio" value="0" ';
			if ($pp_options['originals'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />
			</td>
		</tr>
		<tr>
			<th scope="row">' . __('Originals prefix:','photopress') . '</th>
			<td><input name="origprefix" type="text" id="origprefix" value="' . $pp_options['origprefix'] . '" size="10" /><br />' . __('String to add to the original image, if they are being retained.','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Minimum level to upload and browse:','photopress') . '</th>
			<td><select name="min_level" id="min_level">';
			for ($i = 1; $i <= 10; $i++) {
				echo '
				<option value="' . $i . '"';
				if ($pp_options['min_level'] == $i) { echo ' selected="selected"'; }            echo '>' . $i . '</option>';
			}
			echo '
			</select></td>
		</tr>
		<tr>
			<th scope="row">' . __('Maximum allowed size for uploads:','photopress') . '</th>
			<td><input name="maxk" type="text" id="maxk" value="' . $pp_options['maxk'] . '" size="4" /> 
			' . __('Kilobytes (KB)','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Maximum resized image size (width or height):','photopress') . '</th>
			<td><input name="maxsize" type="text" id="maxsize" value="' . $pp_options['maxsize'] . '" size="4" />
			' . __('pixels','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Maximum thumbnail size:','photopress') . '</th>
			<td><input name="thumbsize" type="text" id="thumbsize" value="' . $pp_options['thumbsize'] . '" size="4" />' . __('pixels','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Quality for resized JPEGs:','photopress') . '</th>
			<td><input name="quality" type="text" id="quality" value="' . $pp_options['quality'] . '" size="3" /><br />' . __('Recommended: between 70 and 95','photopress') . '</td>
		</tr>
		<tr>
			<th valign="top" scope="row">' . __('Allowed file extensions (and MIME types):','photopress') . '</th>
			<td><input name="allowedtypes" type="text" id="allowedtypes" value="' . $pp_options['allowedtypes'] . '" size="40" /><br />' . __('Recommended: <code>jpg jpeg png gif</code> (The resizing function only supports these types. Both jpg and jpeg are necessary because we compare MIME type and extension.)','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Insert thumb or full image by default:','photopress') . '</th>
			<td><label><input name="insert_thumbs" type="radio" value="1" ';
			if ($pp_options['insert_thumb'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Thumb','photopress') . '</label> 
			<label><input name="insert_thumbs" type="radio" value="0" ';
			if ($pp_options['insert_thumb'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('Full','photopress') . '</label><br />' . __('Both options will be in a dropdown, this selects the default.','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Thumbnails prefix:','photopress') . '</th>
			<td><input name="thumbprefix" type="text" id="thumbprefix" value="' . $pp_options['thumbprefix'] . '" size="10" /><br />' . __('String to add to thumbnails Photopress creates.','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Link to album:','photopress') . '</th>
			<td>
			<label><input name="album" type="radio" value="1" ';
			if ($pp_options['album'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label> 
			<label><input name="album" type="radio" value="0" ';
			if ($pp_options['album'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />' . __('The code for inserted images can point to the image in the album or straight to a popup containing the image.','photopress') . '
        </td>
		</tr>
		<tr>
			<th scope="row">' . __('Insert link in Meta:','photopress') . '</th>
			<td>
			<label><input name="meta_link" type="radio" value="1" ';
			if ($pp_options['meta_link'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label> 
			<label><input name="meta_link" type="radio" value="0" ';
			if ($pp_options['meta_link'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />' . __('Adds a link to the photo album in the Meta sidebar list.','photopress') . '
			</td>
		</tr>
		<tr>
			<th scope="row">' . __('Insert random image in Meta:','photopress') . '</th>
			<td>
			<label><input name="meta_rand" type="radio" value="1" ';
			if ($pp_options['meta_rand'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label> 
			<label><input name="meta_rand" type="radio" value="0" ';
			if ($pp_options['meta_rand'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />' . __('Adds a random linked thumb to the Meta sidebar list.','photopress') . '
			</td>
		</tr>
		<tr>
			<th scope="row">' . __('Add failsafe buttons:','photopress') . '</th>
			<td>
			<label><input name="failsafe_buttons" type="radio" value="1" ';
			if ($pp_options['failsafe_buttons'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label> 
			<label><input name="failsafe_buttons" type="radio" value="0" ';
			if ($pp_options['failsafe_buttons'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />' . __('Enables a Photos button that may work more reliably than the default button.','photopress') . '
			</td>
		</tr>
		<tr>
			<th scope="row">' . __('Show posts with image in album:','photopress') . '</th>
			<td>
			<label><input name="show_posts" type="radio" value="1" ';
			if ($pp_options['show_posts'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label> 
			<label><input name="show_posts" type="radio" value="0" ';
			if ($pp_options['show_posts'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />' . __('Shows a list of posts that contain an image in the album. Notes on making this work better with your theme can be found at: ','photopress') . '<a href="http://familypress.net/archives/fixing-the-sidebar/">familypress.net</a>
			</td>
		</tr>
		<tr>
			<th scope="row">' . __('Show thumbs in mass edit in the album manager:','photopress') . '</th>
			<td>
			<label><input name="thumbs_in_mass_edit" type="radio" value="1" ';
			if ($pp_options['thumbs_in_mass_edit'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label> 
			<label><input name="thumbs_in_mass_edit" type="radio" value="0" ';
			if ($pp_options['thumbs_in_mass_edit'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />' . __('Toggles thumbnail display in the Mass Editor part of the album manager.','photopress') . '
			</td>
		</tr>
		<tr>
			<th scope="row">' . __('Allow images used in posts to be deleted:','photopress') . '</th>
			<td>
			<label><input name="allow_post_image_delete" type="radio" value="1" ';
			if ($pp_options['allow_post_image_delete'] == '1') {echo 'checked="checked" ';}
			echo '/> ' . __('Yes','photopress') . '</label>
			<label><input name="allow_post_image_delete" type="radio" value="0" ';
			if ($pp_options['allow_post_image_delete'] == '0') {echo 'checked="checked" ';}
			echo ' /> ' . __('No','photopress') . '</label><br />' . __('Makes the delete button appear in the album manager for images that are used in posts.','photopress') . '
			</td>
		</tr>
		<tr>
			<th scope="row">' . __('Images per page:','photopress') . '</th>
			<td><input name="images_per_page" type="text" id="images_per_page" value="' . $pp_options['images_per_page'] . '" size="3" /><br />' . __('Number of images to show on pages with multiple images. (The album, the browse list, and in the manager.)','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Album columns:','photopress') . '</th>
			<td><input name="album_columns" type="text" id="album_columns" value="' . $pp_options['album_columns'] . '" size="3" /><br />' . __('Number of columns to use in the album: probably 2 if you have a sidebar, maybe 3 otherwise.','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('Manager columns:','photopress') . '</th>
			<td><input name="manager_columns" type="text" id="manager_columns" value="' . $pp_options['manager_columns'] . '" size="3" /><br />' . __('Number of columns to use in the album manager, probably 4 or 5.','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('CSS classes for inserted images:','photopress') . '</th>
			<td><input name="image_class" type="text" id="image_class" value="' . $pp_options['image_class'] . '" size="10" /><br />' . __('alignleft, alignright, and centered are available in the default theme, or you can add new classes to your theme and enter the names here. Separate the classes with spaces. These will appear in a dropdown list in the uploader and browser, and the first one will be the default.','photopress') . '</td>
		</tr>
		<tr>
			<th scope="row">' . __('CSS class for random thumbs:','photopress') . '</th>
			<td><input name="rand_class" type="text" id="rand_class" value="' . $pp_options['rand_class'] . '" size="10" /><br />' . __('alignleft, alignright, and centered are available in the default theme, or you can add a new class to your theme and enter the name here.','photopress') . '</td>
		</tr>
	</table> 
	</fieldset>
	<p class="submit"><input type="submit" name="Submit" value="' . __('Update Options &raquo;','photopress') . '" /></p>
</form> 
</div>
';
}

add_action('admin_menu', 'pp_add_options_page');

// makes the Manage:Photopress Album pages
function pp_album_management() {
	global $wpdb,$pp_options;
	if (isset($_POST['pp_import_orphans'])) {
		$imported = pp_import_orphans();
		echo '<div class="updated">' . $imported . __(' images imported.','photopress') . '</div>';
	}
	if (isset($_POST['pp_reinstall'])) {
		$install = pp_table_install();
		echo '<div class="updated">' . __('Photopress table updated.','photopress') . '</div>';
	}
	if (isset($_POST['pp_change_cat'])) {
		$catrec = 0;
		foreach ((array)$_POST as $key=>$val) {
			if (is_array($val)) {
				if (!empty($val['newcat'])) {
					$update_cat = $val['newcat'];
				} else {
					$update_cat = $val['dropcat'];
				}
				if ($update_cat != $val['oldcat']) {
					if (pp_change_cat($val['oldcat'],$update_cat)) {
						$catrec++;
					}
				}
			}
		}
		$wpdb->flush();
		echo '<div class="updated">' . $catrec . __(' album categories changed.','photopress') . '</div>';
	}
	if (isset($_POST['pp_album_update'])) {
		if(!empty($_POST['newimgcat'])) {
			$pp_updated_cat = $_POST['newimgcat'];
		} else {
			$pp_updated_cat = $_POST['oldimgcat'];
		}
		$pp_updated_array = array($_POST['imgfile'],$_POST['imgname'],$_POST['imgdesc'],$pp_updated_cat);
		if (pp_table_update($pp_updated_array)) {
			echo '<div class="updated">' . __('Photopress album data updated.','photopress') . '</div>';
		} else {
			echo '<div class="updated">' . __('Photopress album data update failed for some reason.','photopress') . '</div>';
		}
	}
	if (isset($_POST['pp_mass_update'])) {
		$dels = 0;
		$updates = 0;
		foreach ((array)$_POST as $key=>$val) {
			if (is_array($val)) {
				if ($val['imgdelete'] == '1') {
					if (pp_delete_photo($val['imgfile'])) {
						$dels++;
					}
				} else {
					if (!empty($val['newimgcat'])) {
						$pp_updated_cat = $val['newimgcat'];
					} else {
						$pp_updated_cat = $val['oldimgcat'];
					}
					$pp_updated_array = array($val['imgfile'],$val['imgname'],$val['imgdesc'],$pp_updated_cat);
					if (pp_table_update($pp_updated_array)) {
						$updates++;
					}
				}
				$wpdb->flush();
			}
		}
		echo '<div class="updated">' . $updates . __(' records updated.','photopress') . $dels . __(' images deleted.','photopress') . '</div>';
	}
	if (isset($_GET['pp_delete'])) {
		if (pp_delete_photo($_GET['pp_delete'])) {
			echo '<div class="updated">' . $_GET['pp_delete'] . __(' was deleted.','photopress') . '</div>';
		} else {
			echo '<div class="updated">' . $_GET['pp_delete'] . __(' was not deleted for some reason.','photopress') . '</div>';
		}
	}
	if (isset($_POST['pp_db_cleanup'])) {
		if ($cleaned = pp_db_cleanup()) {
			echo '<div class="updated">' . $cleaned . __(' records had no matching images and were removed.','photopress') . '</div>';
		} else {
			echo '<div class="updated">' . __('No cleanup necessary.','photopress') . '</div>';
		}
	}
	echo '
	<div class="wrap">
	<h2>' . __('Photopress Album Management','photopress') . '</h2>
	<p>
	';
	if (pp_count() > 0) {
		echo '<a href="edit.php?page=photopress.php" title="' . __('Category View','photopress') . '">' . __('Category View','photopress') . '</a> | 
	<a href="edit.php?page=photopress.php&amp;pp_manage_mass=yes" title="' . __('Mass Edit','photopress') . '">' . __('Mass Edit','photopress') . '</a> | 
	<a href="edit.php?page=photopress.php&amp;pp_change_cat=yes" title="' . __('Edit Categories','photopress') . '">' . __('Edit Categories','photopress') . '</a> | 
		';
	}
	echo '<a href="#" onclick="return pp_popup();" title="' . __('Upload','photopress') . '">' . __('Upload','photopress') . '</a> | 
	<a href="edit.php?page=photopress.php&amp;pp_maintain=yes" title="' . __('Maintain','photopress') . '">' . __('Maintain','photopress') . '</a>
	</p>
	</div>
	';
	$pp_categories = pp_list_cats();
	if (isset($_GET['pp_maintain'])) {
		echo '
		<div class="wrap">
		<h2>' . __('Maintain','photopress') . '</h2>
		<p>' . __('Various possibly-useful maintenance tools. <strong>Import Photos</strong> imports any images in the photos folder that are not in the database. <strong>Install DB</strong> nondestructively updates the database, adding new columns if necessary. <strong>Cleanup DB</strong> checks for images in the DB that are not on disk and removes the records. If you installed the plugin correctly these should not be necessary.','photopress') . '</p>
		<form name="pp_album_cats" method="post">
		<input type="hidden" name="pp_import_orphans" value="update" />
		<p class="submit"><input type="submit" name="Submit" value="' . __('Import Photos','photopress') . ' &raquo;" /></p>
		</form>
		<form name="pp_album_cats" method="post">
		<input type="hidden" name="pp_reinstall" value="update" />
		<p class="submit"><input type="submit" name="Submit" value="' . __('Install DB','photopress') . ' &raquo;" /></p>
		</form>
		<form name="pp_album_cats" method="post">
		<input type="hidden" name="pp_db_cleanup" value="update" />
		<p class="submit"><input type="submit" name="Submit" value="' . __('Cleanup DB','photopress') . ' &raquo;" /></p>
		</form>
		</div>
		';
	} elseif (isset($_GET['pp_manage_image'])) {
		$pp_manage_image = $_GET['pp_manage_image'];
		$pp_image_data = pp_get_data($pp_manage_image);
		$pp_imgfile = $pp_image_data['imgfile'];
		$pp_imgdesc = stripslashes($pp_image_data['imgdesc']);
		$pp_imgname = stripslashes($pp_image_data['imgname']);
		$pp_imgcat = stripslashes($pp_image_data['imgcat']);
		$pp_imgtime = $pp_image_data['imgtime'];
		echo '
		<div class="wrap">
		<h2>' . __('Editing','photopress') . ' <em>' . $pp_imgfile . '</em> ' . __('from the','photopress') . ' <em><a href="edit.php?page=photopress.php&amp;pp_manage_cat=' . urlencode($pp_imgcat) . '">' . $pp_imgcat . '</a></em> ' . __('category','photopress') . '</h2>
		<form name="pp_album_cats" method="post">
		<input type="hidden" name="pp_album_update" value="update" />
		<input type="hidden" name="imgfile" value="' . $pp_imgfile . '" />
		<fieldset class="options">
		<table width="100%" cellpadding="5">
		<tr><td>
		<img src="' . $pp_options['photosaddress'] . '/' . $pp_manage_image . '" alt="' . $pp_imgname . '" title="' . $pp_imgname . '" />
		</td><td>
		<p><strong>' . __('Category:','photopress') . '</strong><br /><select name="oldimgcat" id="pp_oldimgcat">';
		echo pp_cat_dropdown($pp_categories,$pp_imgcat); 
		echo '</select><br />
		 ' . __('or enter new:','photopress') . ' <input type="text" name="newimgcat" value="" /></p>
		<p><strong>' . __('Name:','photopress') . '</strong><br /><input type="text" name="imgname" value="' . $pp_imgname . '" /></p>
		<p><strong>' . __('Description:','photopress') . '</strong><br /><textarea rows="2" cols ="30" name="imgdesc" id="imgdesc" class="uploadform">' . $pp_imgdesc . '</textarea></p>
		<p><strong>' . __('Date uploaded:','photopress') . '</strong><br />' . date("j M Y, H:i", $pp_imgtime) . '</p>
		<p class="submit">
		<input type="submit" name="Submit" value="' . __('Update','photopress') . ' &raquo;" />
		</p>';
		query_posts("s=$pp_manage_image");
		if ( have_posts() ) :
		echo '<p>' . __('Posts with this image:','photopress') . '<br />';
		while ( have_posts() ) : the_post();
		echo ' <a href="';
		the_permalink();
		echo '">';
		the_title();
		echo '</a> ';
		endwhile;
		echo '</p>';
		else:
		if ($pp_options['allow_post_image_delete'] == '0') {
			echo "<p><a href=\"edit.php?page=photopress.php&amp;pp_manage_cat=" . stripslashes($pp_image_data['imgcat']) . "&pp_delete=" . $pp_manage_image . "\" class=\"delete\" onclick=\"return confirm('" . __("OK to delete ","photopress") . $pp_manage_image . "?')\">" . $pp_manage_image . __(" isn't used in any posts. Delete?","photopress") . "</a></p>\n";
		}
		endif;
		if ($pp_options['allow_post_image_delete'] != '0') {
			echo "<p><a href=\"edit.php?page=photopress.php&amp;pp_manage_cat=" . stripslashes($pp_image_data['imgcat']) . "&pp_delete=" . $pp_manage_image . "\" class=\"delete\" onclick=\"return confirm('" . __("OK to delete ","photopress") . $pp_manage_image . "?')\">" . __("Delete ","photopress") . $pp_manage_image . "?</a></p>\n";
		}
		echo '
		</td></tr></table>
		</fieldset>
		</form>
		</div>
		';
	} elseif (isset($_GET['pp_manage_mass'])) {
		if (isset($_GET['pp_page'])) {
			$current_page = $_GET['pp_page'];
		} else {
			$current_page = 1;
		}
		echo '
		<div class="wrap">
		<h2>' . __('Mass Edit','photopress') . '</h2>
		<p><form>' . __('Sort: ','photopress') . '<select name="sort" onChange="setbrowsesort(this)">
			<option value="pp_manage_mass=yes&sort=imgtimeD"'; if ($_GET['sort'] == 'imgtimeD' || !isset($_GET['sort'])) { echo ' selected="selected"'; } echo '>' . __('New to Old','photopress') . '</option>
			<option value="pp_manage_mass=yes&sort=imgtimeA"'; if ($_GET['sort'] == 'imgtimeA') { echo ' selected="selected"'; } echo '>' . __('Old to New','photopress') . '</option>
			<option value="pp_manage_mass=yes&sort=imgfileA"'; if ($_GET['sort'] == 'imgfileA') { echo ' selected="selected"'; } echo '>' . __('A to Z','photopress') . '</option>
			<option value="pp_manage_mass=yes&sort=imgfileD"'; if ($_GET['sort'] == 'imgfileD') { echo ' selected="selected"'; } echo '>' . __('Z to A','photopress') . '</option>
			<option value="pp_manage_mass=yes&sort=imgcatA"'; if ($_GET['sort'] == 'imgcatA') { echo ' selected="selected"'; } echo '>' . __('A-Z by cat','photopress') . '</option>
			<option value="pp_manage_mass=yes&sort=imgcatD"'; if ($_GET['sort'] == 'imgcatD') { echo ' selected="selected"'; } echo '>' . __('Z to A by cat','photopress') . '</option>
		</select></form></p>';
		if ($_GET['sort'] == 'imgfileD') {
			$pp_sort = 'imgfile DESC';
		} elseif ($_GET['sort'] == 'imgfileA') {
			$pp_sort = 'imgfile';
		} elseif ($_GET['sort'] == 'imgtimeA') {
			$pp_sort = 'imgtime';
		} elseif ($_GET['sort'] == 'imgcatA') {
			$pp_sort = 'imgcat';
		} elseif ($_GET['sort'] == 'imgcatD') {
			$pp_sort = 'imgcat DESC';
		} else {
			$pp_sort = 'imgtime DESC';
		}
		$image_count = pp_count();
		$pages = (int)ceil(($image_count/$pp_options['images_per_page']));
		echo '<p>';
		if ($pages > 1) {
			for ($i = 1; $i <= $pages; $i++) {
				if ($i != $current_page) {
					echo '<a href="edit.php?page=photopress.php&amp;pp_manage_mass=yes&amp;pp_page=' . $i; if (isset($_GET['sort'])) { echo '$amp;pp_sort=' . $_GET['sort']; } echo '">';
				}
				echo __('Page','photopress') . ' ' . $i;
				if ($i != $current_page) {
					echo '</a>';
				}
				if ($i < $pages) {
					echo ' | ';
				}
			}
			$list_start = ($current_page - 1) * $pp_options['images_per_page'];
		} else {
			$list_start = 0;
		}
		echo '</p>';
		echo '<form name="pp_album_cats" method="post">
		<input type="hidden" name="pp_mass_update" value="update" />
		<table>
		<tr>
		<th scope="col">' . __('File','photopress') . '</th>
		<th colspan="2">' . __('Category','photopress') . '</th>
		<th scope="col">' . __('Name','photopress') . '</th>
		<th scope="col">' . __('Description','photopress') . '</th>
		<th scope="col">' . __('Delete','photopress') . '</th>
		</tr>
		';
		$current_image_list = pp_images_with_data('',$pp_sort,$list_start,$pp_options['images_per_page']);
		$i = 1;
		foreach ((array)$current_image_list as $image_data) {
			if (strlen($image_data['imgfile']) > 20) {
				$image_file_short = substr($image_data['imgfile'],0,17) . '...';
			} else {
				$image_file_short = $image_data['imgfile'];
			}
			echo '<tr>
			<td><input type="hidden" name="row' . $i . '[imgfile]" value="' . $image_data['imgfile'] . '" />';
			if ($pp_options['thumbs_in_mass_edit'] == '1') {
				$thumb_size = getimagesize($pp_options['photospath'] . '/' . $pp_options['thumbprefix'] . $image_data['imgfile']);
				echo '<img src="' . $pp_options['photosaddress'] . '/' . $pp_options['thumbprefix'] . $image_data['imgfile'] . '" ' . $thumb_size[3] . ' /><br />';
			}
			echo $image_file_short . '</td>
			<td><select name="row' . $i . '[oldimgcat]">' . pp_cat_dropdown($pp_categories,stripslashes($image_data['imgcat'])) . '</select></td>
			<td><input type="text" size="12" name="row' . $i . '[newimgcat]" value="" /></td>
			<td><input type="text" size="12" name="row' . $i . '[imgname]" value="' . stripslashes($image_data['imgname']) . '" /></td>
			<td><input type="text" size="27" name="row' . $i . '[imgdesc]" value="' . stripslashes($image_data['imgdesc']) . '" /></td>
			<td><input type="checkbox" name="row' . $i . '[imgdelete]" value="1" /></td>
			</tr>
			';
			$i++;
		}
		echo '
		</table>
		<p class="submit"><input type="submit" name="Submit" value="' . __('Update','photopress') . ' &raquo;" /></p>
		</form>
		</div>
		';
	} elseif (isset($_GET['pp_manage_cat'])) {
		$pp_manage_cat = html_entity_decode($_GET['pp_manage_cat']);
		$stripped_cat = stripslashes($pp_manage_cat);
		echo '
		<div class="wrap">
		<h2>' . __('Images in the','photopress') . ' <em>' . $stripped_cat . '</em> ' . __('category','photopress') . '</h2>
   	<p><form>' . __('Sort: ','photopress') . '<select name="sort" onChange="setbrowsesort(this)">
			<option value="pp_manage_cat=' . $stripped_cat . '&sort=imgtimeD"'; if ($_GET['sort'] == 'imgtimeD' || !isset($_GET['sort'])) { echo ' selected="selected"'; } echo '>' . __('New to Old','photopress') . '</option>
			<option value="pp_manage_cat=' . $stripped_cat . '&sort=imgtimeA"'; if ($_GET['sort'] == 'imgtimeA') { echo ' selected="selected"'; } echo '>' . __('Old to New','photopress') . '</option>
			<option value="pp_manage_cat=' . $stripped_cat . '&sort=imgfileA"'; if ($_GET['sort'] == 'imgfileA') { echo ' selected="selected"'; } echo '>' . __('A to Z','photopress') . '</option>
			<option value="pp_manage_cat=' . $stripped_cat . '&sort=imgfileD"'; if ($_GET['sort'] == 'imgfileD') { echo ' selected="selected"'; } echo '>' . __('Z to A','photopress') . '</option>
			</select></form></p>';
		if ($_GET['sort'] == 'imgfileD') {
			$pp_sort = 'imgfile DESC';
		} elseif ($_GET['sort'] == 'imgfileA') {
			$pp_sort = 'imgfile';
		} elseif ($_GET['sort'] == 'imgtimeA') {
			$pp_sort = 'imgtime';
		} else {
			$pp_sort = 'imgtime DESC';
		}
		if (isset($_GET['pp_page'])) {
			$current_page = $_GET['pp_page'];
		} else {
			$current_page = 1;
		}
		$image_count = pp_count($pp_manage_cat);
		$pages = (int)ceil($image_count/$pp_options['images_per_page']);
		if ($pages > 1) {
			echo '<p>';
			for ($i=1; $i<=$pages; $i++) {
				if ($i != $current_page) {
					echo '<a href="' . get_settings('siteurl') . '/wp-admin/edit.php?page=photopress.php&amp;pp_manage_cat=' . $pp_manage_cat . '&amp;pp_page=' . $i; if (isset($_GET['sort'])) { echo '&amp;sort=' . $_GET['sort']; } echo '">';
				}
				echo __('Page','photopress') . ' ' . $i;
				if ($i != $current_page) {
					echo '</a>';
				}
				if ($i < $pages) {
					echo ' | ';
				}
			}
			echo '</p>
			';
			$list_start = ($current_page - 1) * $pp_options['images_per_page'];
		} else { // there's only one page so it's the whole list
				$list_start = 0;
		}
		echo '<table>
		';
		$current_image_list = pp_images_with_data($pp_manage_cat,$pp_sort,$list_start,$pp_options['images_per_page']);
		$column = 0;
		foreach((array)$current_image_list as $pp_image) {
			$column++;
			if ($column == 1) {
				echo '<tr>
				';
			}
			echo '<td><a href="edit.php?page=photopress.php&amp;pp_manage_image=' . $pp_image['imgfile'] . '" title="' . stripslashes($pp_image['imgname']). '"><img src="' . $pp_options['photosaddress'] . '/' . $pp_options['thumbprefix'] . $pp_image['imgfile'] . '" title="' . stripslashes($pp_image['imgname']) . '" alt="' . stripslashes($pp_image['imgname']) . '" /><br />';
			if (strlen(stripslashes($pp_image['imgname'])) > 15) {
				echo substr(stripslashes($pp_image['imgname']),0,12) . '...';
			} else {
				echo stripslashes($pp_image['imgname']);
			}
			echo '</a></td>
			';
			if ($column == $pp_options['manager_columns']) {
				echo '</tr>
				';
				$column = 0;
			}
		}
		if ($column < $pp_options['manager_columns'] && $column > 0) {
			echo '</tr>
			';
		}
		echo '
		</table>
		';
	} elseif (isset($_GET['pp_change_cat'])) {
		echo '<div class="wrap">
		<h2>' . __('Edit Categories','photopress') . '</h2>
		<form name="pp_album_cats" method="post">
		<input type="hidden" name="pp_change_cat" value="update" />
		<table>
		';
		$i = 0;
		foreach ((array)$pp_categories as $cat) {
			$pp_cat = stripslashes($cat);
			echo '<tr><td><input type="hidden" name="row' . $i . '[oldcat]" value="' . $pp_cat . '" />' . $pp_cat . '</td>
			<td><select name="row' . $i . '[dropcat]" id="dropcat">';
			echo pp_cat_dropdown($pp_categories,$pp_cat); 
			echo '</select></td>
			<td><input type="text" value="" name="row' . $i . '[newcat]" /></td></tr>
			';
			$i++;
		}
		echo '</table>
			<p class="submit"><input type="submit" name="Submit" value="' . __('Update Categories &raquo;','photopress') . '" /></p>
		</div>
		';
	} elseif (pp_count() > 0) {
		echo '<div class="wrap">
		<h2>' . __('Categories','photopress') . '</h2>
		<table>
		';
		$column = 0;
		foreach ((array)$pp_categories as $cat) {
			$pp_cat = stripslashes($cat);
			$column++;
			if ($column == 1) {
				echo '<tr>';
			}
			echo '
			<td><a href="edit.php?page=photopress.php&amp;pp_manage_cat=' . htmlentities2($pp_cat) . '">' . pp_random_image($cat,2) . '<br />
			' . $pp_cat . ' (' . pp_count($cat) . ')</a></td>
			';
			if ($column == $pp_options['manager_columns']) {
				echo '</tr>
				';
				$column = 0;
			}
		}
		if ($column > 0) {
			echo '</tr>
			';
		}
		echo '</table>
		</div>
		';
	}
}

function pp_add_management_page() {
	global $pp_options;
	add_management_page('Photopress', 'Photopress', $pp_options['min_level'], basename(__FILE__), 'pp_album_management');
}

add_action('admin_menu', 'pp_add_management_page');

// delete function, not sure how secure it is...
function pp_delete_photo($photo) {
	global $pp_options, $wpdb, $table_prefix;
	$pp_phototodelete = $pp_options['photospath'] . '/' . $photo;
	$pp_thumbtodelete = $pp_options['photospath'] . '/' . $pp_options['thumbprefix'] . $photo;
	$pp_origtodelete = $pp_options['photospath'] . '/' . $pp_options['origprefix'] . $photo;
	$table_name = $table_prefix . "photopress";
	if (!is_file($pp_thumbtodelete) || !is_file($pp_phototodelete)) {
		$wpdb->query("DELETE FROM $table_name WHERE imgfile = '$photo'");
		return FALSE;
	} else {
		@unlink($pp_phototodelete);
		@unlink($pp_thumbtodelete);
		if (is_file($pp_origtodelete)) {
			@unlink($pp_origtodelete);
		}
		$wpdb->query("DELETE FROM $table_name WHERE imgfile = '$photo'");
		return TRUE;
	}
	
}

// if the failsafe buttons are turned on in Options:Photopress, let's insert them
if ($pp_failsafe_buttons == '1') {
	function pp_failsafe_buttons() {
		echo '
			<input type="button" value="Upload" onclick="return pp_upload_popup()" /> <input type="button" value="Browse" onclick="return pp_browse_popup()" />';
	}
	add_action('simple_edit_form', 'pp_failsafe_buttons');
	add_action('edit_form_advanced', 'pp_failsafe_buttons');
}

// install a table in the db for PP image data
function pp_table_install() {
	global $table_prefix, $wpdb, $user_level;
	$table_name = $table_prefix . "photopress";
	get_currentuserinfo();
	if ($user_level < 8) { return; }
	$sql = "CREATE TABLE ".$table_name." (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		imgfile varchar(100) NOT NULL,
		imgname varchar(55) NULL,
		imgdesc text NULL,
		imgcat varchar(55) NULL,
		imgtime varchar(55) NULL,
		UNIQUE KEY id (id)
	);";
	require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
	dbDelta($sql);
// import image info stored in the WP Options by previous version of PP
	if (get_option('pp_album_cats') && !is_array($wpdb->get_col("SELECT imgfile FROM $table_name LIMIT 5"))) { // only import if there's data to import and the table is empty
		$images = array();
		$images = pp_folder_contents();
		$cat_array = array();
		$cat_array = get_option('pp_album_cats');
		foreach((array)$images as $img) {
			$img_key = substr_replace($img, '_', -4, 1);
			if (!empty($cat_array[$img_key])) {
				$category = $cat_array[$img_key];
			} else {
				$category = 'Default';
			}
			$name = strtr(substr($img, 0, -4), "_", " ");
			$import_array = array($img, $name, $name, $category);
			pp_table_update($import_array);
		}
	}

}

// run the table installer when the plugin is activated
if (isset($_GET['activate']) && $_GET['activate'] == 'true') {
	add_action('init', 'pp_table_install');
}

// imports images in the photos folder that aren't in the database
function pp_import_orphans() {
	global $wpdb, $table_prefix;
	$table_name = $table_prefix . 'photopress';
	$full_array = pp_folder_contents();
	$in_db = $wpdb->get_col("SELECT imgfile FROM $table_name");
	$not_in_db = array_diff($full_array, $in_db);
	$imported = 0;
	foreach((array)$not_in_db as $imgfile) {
		$imgname = strtr(substr($imgfile, 0, strrpos($imgfile, '.')), "_", " ");
		$imgcat = 'Default';
		$import_array = array($imgfile, $imgname, $imgname, $imgcat);
		pp_table_update($import_array);
		$imported++;
	}
	return $imported;
}

function pp_change_cat($oldcat,$newcat) {
	global $wpdb, $table_prefix, $pp_options;
	$table_name = $table_prefix . 'photopress';
	$oldcat = $wpdb->escape($oldcat);
	$newcat = $wpdb->escape($newcat);
	if ($wpdb->query("UPDATE $table_name SET imgcat='$newcat' WHERE binary imgcat='$oldcat'")) {
		return TRUE;
	} else {
		return FALSE;
	}
}

function pp_db_cleanup() {
	global $wpdb, $table_prefix, $pp_options;
	$table_name = $table_prefix . 'photopress';
	$full_array = pp_folder_contents();
	$in_db = $wpdb->get_col("SELECT imgfile FROM $table_name");
	$not_on_disk = array_diff($in_db, $full_array);
	$records = 0;
	foreach ($not_on_disk as $photo) {
		if (!is_file($pp_options['photospath'] . '/' . $photo)) {
			if ($wpdb->query("DELETE FROM $table_name WHERE imgfile = '$photo'")) {
				$records++;
			}
		}
	}
	if ($records > 0) {
		return $records;
	} else {
		return FALSE;
	}
}

// turn a PP tag into a linked image tag
function photopress_make_link($stuff) {
	global $pp_options;
	$file = explode(',',$stuff);
	if (!empty($file[2])) {
		$image_class = $file[2];
	} else {
		$image_class = 'centered';
	}
	if ($file[1] == 'thumb') {
		$thumbsize = getimagesize($pp_options['photospath'] . '/' . $pp_options['thumbprefix'] . $file[0]);
		$image_data = pp_get_data($file[0]);
		return '<a href="' . htmlentities2($pp_options['imgaddress'] . $file[0]) . '" title="' . $image_data['imgname'] . '"><img src="' . $pp_options['photosaddress'] . '/' . $pp_options['thumbprefix'] . $file[0] . '" class="' . $image_class . '" alt="' . $image_data['imgname'] . '" ' . $thumbsize[3] . ' /></a>';
	} elseif ($file[1] == 'full') {
		$image_data = pp_get_data($file[0]);
		$fullsize = getimagesize($pp_options['photospath'] . '/' . $file[0]);
		return '<a href="' . htmlentities2($pp_options['imgaddress'] . $file[0]) . '" title="' . $image_data['imgname'] . '"><img src="' . $pp_options['photosaddress'] . '/' . $file[0] . '" class="' . $image_class . '" alt="' . $image_data['imgname'] . '" ' . $fullsize[3] . ' /></a>';
	} else {
		return $file[0];
	}
}

// process WP content for PP tags
function photopress_tag_process($content = '') {
	return preg_replace( "/\[photopress\:(.*)\]/e","photopress_make_link('\\1')", $content);
}

add_filter('the_content', 'photopress_tag_process');

function photopress_album_rewrite($rewrite) {
	global $wp_rewrite, $pp_options;
	$wp_rewrite->add_rewrite_tag('%image%', '(.+)', 'pp_album=main&pp_image=');
	$structure = $wp_rewrite->root . $pp_options['album_token'] . '/' . $pp_options['images_token'] . '/%image%';
	$keywords_rewrite = $wp_rewrite->generate_rewrite_rules($structure);
	$wp_rewrite->add_rewrite_tag('%cat%', '(.+)', 'pp_album=main&pp_cat=');
	$wp_rewrite->add_rewrite_tag('%ppage%', '(.+)', 'pp_page=');
	$structure = $wp_rewrite->root . $pp_options['album_token'] . '/' . $pp_options['cat_token'] . '/%cat%/%ppage%';
	$keywords_rewrite2 = $wp_rewrite->generate_rewrite_rules($structure);
	$wp_rewrite->add_rewrite_tag('%album%', '(' . $pp_options['album_token'] . ')', 'pp_album=main');
	$structure = $wp_rewrite->root . '%album%';
	$keywords_rewrite3 = $wp_rewrite->generate_rewrite_rules($structure);
	return ($rewrite + $keywords_rewrite + $keywords_rewrite2 + $keywords_rewrite3);
}

if ($pp_options['use_permalinks'] == '1' && isset($wp_rewrite) && $wp_rewrite->using_permalinks()) {
	add_filter('search_rewrite_rules', 'photopress_album_rewrite');
}
?>
